"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.filter = void 0;
const retil_support_1 = require("retil-support");
const fuse_1 = require("./fuse");
/**
 * Make the source valueless when the current value doesn't match the given
 * filter function.
 *
 * Use with `mergeLatest` to create a source that keeps the latest matching
 * value.
 */
function filter(source, predicate) {
    return fuse_1.fuse((use) => {
        const value = use(source);
        if (!predicate(value)) {
            // Throw a never-ending promise to wait for the next value.
            // eslint-disable-next-line no-throw-literal
            throw {
                then: retil_support_1.noop,
            };
        }
        return value;
    });
}
exports.filter = filter;
