"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useSourceLegacy = void 0;
const react_1 = require("react");
const use_subscription_1 = require("use-subscription");
const source_1 = require("../source");
const MissingToken = Symbol();
const useSourceLegacy = (maybeSource, options = {}) => {
    const hasDefaultValue = 'defaultValue' in options;
    const { defaultValue, startTransition } = options;
    const [core, inputSelect] = maybeSource || source_1.nullSource;
    const select = react_1.useCallback(([getVersion]) => hasDefaultValue && !source_1.hasSnapshot([[getVersion], inputSelect])
        ? MissingToken
        : inputSelect(getVersion()), [hasDefaultValue, inputSelect]);
    const getCurrentValue = react_1.useCallback(() => select(core), [core, select]);
    const subscribe = react_1.useMemo(() => startTransition
        ? (callback) => core[1](() => {
            startTransition(callback);
        })
        : core[1], [core, startTransition]);
    const subscription = react_1.useMemo(() => ({
        getCurrentValue,
        subscribe,
    }), [getCurrentValue, subscribe]);
    // useSubscription doesn't suspend if its value changes to a missing value.
    // This hack normalizes it to act the same as useMutableSource.
    const [, setState] = react_1.useState();
    react_1.useEffect(() => {
        if (!hasDefaultValue) {
            return subscribe(() => {
                if (!source_1.hasSnapshot([core])) {
                    setState((() => {
                        // Because hasSnapshot returns false, this'll throw a promise.
                        select(core);
                    }));
                }
            });
        }
    }, [hasDefaultValue, core, select, subscribe]);
    const value = use_subscription_1.useSubscription(subscription);
    return value === MissingToken || maybeSource === null ? defaultValue : value;
};
exports.useSourceLegacy = useSourceLegacy;
