"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.wait = void 0;
const source_1 = require("./source");
const wait = (source, maybePredicate) => {
    // Don't wait for a predicate that already matches.
    if (maybePredicate &&
        source_1.hasSnapshot(source) &&
        maybePredicate(source_1.getSnapshot(source))) {
        return;
    }
    // By default, any snapshot other than the current value will do.
    const predicate = maybePredicate || (() => true);
    // Get a promise that resolves once the source's snapshot matches the
    // condition.
    return new Promise((resolve, reject) => {
        const unsubscribe = source_1.subscribe(source, () => {
            if (source_1.hasSnapshot(source)) {
                try {
                    if (predicate(source_1.getSnapshot(source))) {
                        unsubscribe();
                        resolve();
                    }
                }
                catch (error) {
                    reject(error);
                }
            }
        });
    });
};
exports.wait = wait;
